==> IceWarpServer-9.3.2_RHEL4.tar.gz: html/webmail/client/inc/obj_select.js <==
_me = obj_select.prototype;
function obj_select(){};

	_me.__constructor = function(){
		var me = this;

		/* options visibility */
        this.__visibleOptions = false;
        this.__idTable = {};
		this.__tempValue = '';
        this.__disabled = false;
        
		/* set whole object unselectable for Mozilla, Safari, KHTML, MSIE */
		setSelectNone(this._main);

		this.__eButton  = this._getAnchor('button');
		this.__eButton.onmousemove = function(e){
			if(me.__disabled) return false;
			if(this.className.indexOf('obj_selectbuttonhover')== -1) addcss(this,'obj_selectbuttonhover');
		};
		this.__eButton.onmouseout = function(e){
			if(this.className.indexOf('obj_selectbuttonhover')>-1) removecss(this,'obj_selectbuttonhover');
		};

		this.__eButton.onclick = function(e){

  			if (me.__disabled) return false;
  			
			if (me.__visibleOptions)
				me.__hide();
			else
				me.__show();

			return false;
		};

		this.__eButton.onfocus = function(e){
			if (me._onfocus)
				me._onfocus(e);

			me.__exeEvent('onfocus',null,{"owner":this});
		};

		this.__eButton.onblur = function(e){
			me.__hideTimeout = window.setTimeout('try{'+me._pathName + '.__hide();}catch(err){}',150);
		};

		this.__eButton.onkeydown = function(e){

			if(me.__disabled) return false;

			var e = e || window.event;
			var elm = e.target || e.srcElement;

			// e.cancelBubble = true;if (e.stopPropagation) e.stopPropagation();

			//prepare ids
			if (e.keyCode>36 && e.keyCode<41){

				if (!me.__tempValue.toString().length) me.__tempValue = me.__value;
				if (!me.__tempValue.toString().length) return false;

				var before,myself,after;
				for (var i in me.__idTable){
					if  (i != me.__tempValue){
						if  (typeof myself == 'undefined')
							before = i;
						else{
							after = i;
							break;
						}
					}
					else
						myself = i;
				}
			}

			switch(e.keyCode){
			//Up
			case 37:
			case 38:
					if (typeof myself != 'undefined' && typeof before != 'undefined') me._value(before);
					return false;
			//Down
			case 39:
			case 40:
					if (typeof myself != 'undefined' && typeof after != 'undefined') me._value(after);
					return false;
			//Enter
			case 13:
					//hide by onclick event
					break;
			//Esc
			case 27:
					me.__hide();
					break;
			}
		};


	};


	_me.__createOptionList = function(){
		if (this.__eOptions) return;

		this.__eOptions = mkElement('div');
        document.getElementsByTagName('body')[0].appendChild(this.__eOptions);

		this.__eOptions.id = this._pathName + '.options';
		this.__eOptions.className = 'obj_select_options';
		setSelectNone(this.__eOptions);

		this._add_destructor('__destructor',this._pathName);

		var me = this;
		this.__eOptions.onmousedown = function(){

			var e = e || window.event;
			var elm = e.target || e.srcElement;

			switch(elm.tagName){
				case 'A'	: break;
				case 'SPAN'	: elm = elm.parentNode; break;
				default		: return false;
			}

			var id = elm.getAttribute('id').substr(me._pathName.length);

			me._value(id);
			me.__hide();
		};


		this.__eOptions.onmouseover = function(e){
			var e = e || window.event;
			var elm = e.target || e.srcElement;

			if (elm.tagName != 'A') return;

			var id = elm.getAttribute('id').substr(me._pathName.length);

			if (typeof me.__tempValue != 'undefined'){

				if (me.__tempValue == id) return false;

				var elm = document.getElementById(me._pathName+me.__tempValue);
				if (elm) removecss(elm,'active');
			}

			me.__activate(id);

			me.__tempValue = id;
		};
		
		if (this.__idTable)
		    this._fill();
	};
	


	_me.__destructor = function(x){
		if (this.__eOptions) this.__eOptions.parentNode.removeChild(this.__eOptions);
	};
	
	_me.__show = function(){

		this.__createOptionList();

		if (this.__hideTimeout) window.clearTimeout(this.__hideTimeout);
        
		this.__visibleOptions = true;

		var pos_scroll = this._getScrollPosition();
		var pos = getSize(this._main);
  			pos.x -= pos_scroll.x;
			pos.y -= pos_scroll.y;

		this.__eOptions.style.width = pos.w + 'px';
		this.__eOptions.style.top   = (pos.y+pos.h) + 'px';
		this.__eOptions.style.left 	= pos.x + 'px';

		this.__eOptions.className = this._main.className + ' obj_select_options';

		this.__eButton.focus();

		this.__eOptions.style.display = 'block';

		if (count(this.__idTable)>10){
            this.__eOptions.style.overflow = 'auto';

            var size = 0;
			try{
            	size = getSize(this.__eOptions.getElementsByTagName('A')[0]).h*10;
			}
			catch(er){
				size = 153;
			}

			this.__eOptions.style.height = size +'px';
		}
		else{
            this.__eOptions.style.overflow = 'visible';
            this.__eOptions.style.height = 'auto';
		}

		this.__activate();

	};
	
	_me.__hide = function(){
		this.__visibleOptions = false;

		if (this.__eOptions)
			this.__eOptions.style.display = 'none';
	    
	    var elm;
		if((elm = document.getElementById(this._pathName+this.__tempValue))) removecss(elm,'active');
	};
	
	_me._getTextValue = function() {
		if (typeof this.__idTable[this.__value] == 'object')
		    return this.__idTable[this.__value][0];
		else
			return this.__idTable[this.__value];
	};
	/**
	 * atention in obj_charset is the same function!!
	 **/
	_me._value = function (sData,bNoEvn){
		// return
		if (typeof sData == 'undefined') return this.__value;

		var data = typeof this.__idTable[sData] == 'object'?this.__idTable[sData][0]:this.__idTable[sData];
			text = Is.String(data)?data.entityify():data;
		
		// set
        if (typeof data == 'undefined' || (this.__value == sData && this.__eButton.innerHTML == text)) return;
		this.__eButton.innerHTML = '<span>'+text+'</span>';

		this.__activate(sData);
		this.__value = sData;
		
		//inspect2({value:this.__value,type:typeof this.__value});

		if (typeof this.__idTable[sData] == 'object')
        	this.__eButton.className = 'obj_selectbutton '+this.__idTable[sData][1];

		if (!bNoEvn){
			// onchange event
	        if(this._onchange) this._onchange(null);
		    this.__exeEvent('onchange',null,{"owner":this});
	    }
	};
	
	_me.__activate = function(key){
		var elm;
		if  (this.__tempValue){
			if ((elm = document.getElementById(this._pathName+this.__tempValue)))
				removecss(elm,'active');
			this.__tempValue = '';
		}
        
		if (this.__value && (elm = document.getElementById(this._pathName+this.__value))){
			if (typeof key == 'undefined'){
				addcss(elm,'active');

				// scroll to value
				if (elm.offsetTop>this.__eOptions.scrollTop+this.__eOptions.clientHeight || elm.offsetTop<this.__eOptions.scrollTop)
					this.__eOptions.scrollTop = elm.offsetTop - (this.__eOptions.clientHeight/2);

				return;
			}
			else
				removecss(elm,'active');
		}

		if (typeof key != 'undefined' && (elm = document.getElementById(this._pathName+key))){
			addcss(elm,'active');

			// scroll to value
			if (elm.offsetTop<this.__eOptions.scrollTop)
				this.__eOptions.scrollTop = elm.offsetTop+1;
			else
			if (elm.offsetTop+elm.offsetHeight>this.__eOptions.clientHeight+this.__eOptions.scrollTop)
				this.__eOptions.scrollTop = elm.offsetTop+elm.offsetHeight - this.__eOptions.clientHeight-1;
		}
	};
	
	_me._fill = function(aData){
		/* prepare data */
		if (aData)
			this.__idTable = aData

		if (this.__eOptions)
			this.__eOptions.innerHTML = '';

		if (!count(this.__idTable) || !this.__eOptions)
			return false;

		/* fill options */
		var eTmp,sTmp,title;
		
		for (var i in this.__idTable){

			eTmp = mkElement('a',{"href":''});
			
			if (typeof this.__idTable[i] == 'object'){
				title = this.__idTable[i][0];
				eTmp.className = this.__idTable[i][1];
			}
			else
				title = this.__idTable[i];

            eTmp.title = Is.String(title)?title.entityify():title;
			eTmp.innerHTML = '<span>'+(title?title.escapeHTML():'&nbsp;')+'</span>';
			
			eTmp.setAttribute('id',this._pathName+i);

			this.__eOptions.appendChild(eTmp);
			eTmp = null;
		}
		
		/* activate row */
		this.__activate();
	};

	_me._fillLang = function(aData){
		this.__idTable = {};
        for (var i in aData) {
			if (typeof aData[i] == 'object')
			    this.__idTable[i] = [getLang(aData[i][0]),aData[i][1]];
			else
				this.__idTable[i] = getLang(aData[i]);
		}
		this._fill();
	};

	_me._disabled = function(b){
		this.__disabled = (b?true:false);

		if (this.__disabled){
			addcss(this.__eButton,'obj_selectdisabled');
			this.__hide();
		}
		else
			removecss(this.__eButton,'obj_selectdisabled');
	};

	_me.__update = function (sDataSet){
		if (!sDataSet) return;

		if (this._listener == sDataSet)
			this._value(dataSet.get(this._listener,this._listenerPath));
		else
		if (this._listener_data == sDataSet)
			this._fill(dataSet.get(this._listener_data,this._listenerPath_data));
	};

	_me._listen_data = function(sDataSet,aDataPath){
		this._listener_data = sDataSet;
		if (typeof aDataPath == 'object') this._listenerPath_data = aDataPath;
		dataSet.obey(this,'_listener_data',sDataSet);
	};

==> IceWarpServer-9.4.2_RHEL4.tar.gz: html/webmail/client/inc/obj_select.js <==
_me = obj_select.prototype;
function obj_select(){};

	_me.__constructor = function(){
		var me = this;

		/* options visibility */
        this.__visibleOptions = false;
        this.__idTable = {};
		this.__tempValue = '';
        this.__disabled = false;
        
		/* set whole object unselectable for Mozilla, Safari, KHTML, MSIE */
		setSelectNone(this._main);

		this.__eButton  = this._getAnchor('button');
		this.__eButton.onmousemove = function(e){
			if(me.__disabled) return false;
			if(this.className.indexOf('obj_selectbuttonhover')== -1) addcss(this,'obj_selectbuttonhover');
		};
		this.__eButton.onmouseout = function(e){
			if(this.className.indexOf('obj_selectbuttonhover')>-1) removecss(this,'obj_selectbuttonhover');
		};

		this.__eButton.onclick = function(e){

  			if (me.__disabled) return false;
  			
			if (me.__visibleOptions)
				me.__hide();
			else
				me.__show();

			return false;
		};

		this.__eButton.onfocus = function(e){
			if (me._onfocus)
				me._onfocus(e);

			me.__exeEvent('onfocus',null,{"owner":this});
		};

		this.__eButton.onblur = function(e){
			me.__hideTimeout = window.setTimeout('try{'+me._pathName + '.__hide();}catch(err){}',150);
		};

		this.__eButton.onkeydown = function(e){

			if(me.__disabled) return false;

			var e = e || window.event;
			var elm = e.target || e.srcElement;

			// e.cancelBubble = true;if (e.stopPropagation) e.stopPropagation();

			//prepare ids
			if (e.keyCode>36 && e.keyCode<41){

				if (!me.__tempValue.toString().length) me.__tempValue = me.__value;
				if (!me.__tempValue.toString().length) return false;

				var before,myself,after;
				for (var i in me.__idTable){
					if  (i != me.__tempValue){
						if  (typeof myself == 'undefined')
							before = i;
						else{
							after = i;
							break;
						}
					}
					else
						myself = i;
				}
			}

			switch(e.keyCode){
			//Up
			case 37:
			case 38:
					if (typeof myself != 'undefined' && typeof before != 'undefined') me._value(before);
					return false;
			//Down
			case 39:
			case 40:
					if (typeof myself != 'undefined' && typeof after != 'undefined') me._value(after);
					return false;
			//Enter
			case 13:
					//hide by onclick event
					break;
			//Esc
			case 27:
					me.__hide();
					break;
			}
		};


	};


	_me.__createOptionList = function(){
		if (!this.__idTable || !count(this.__idTable))
			return false;

		if (this.__eOptions)
			return true;
			
		this.__eOptions = mkElement('div');
		this.__eOptions.id = this._pathName + '.options';
		this.__eOptions.className = this._main.className + ' obj_select_options';

        document.getElementsByTagName('body')[0].appendChild(this.__eOptions);

		setSelectNone(this.__eOptions);

		this._add_destructor('__destructor',this._pathName);

		var me = this;
		this.__eOptions.onmousedown = function(){

			var e = e || window.event;
			var elm = e.target || e.srcElement;

			switch(elm.tagName){
				case 'A'	: break;
				case 'SPAN'	: elm = elm.parentNode; break;
				default		: return false;
			}

			var id = elm.getAttribute('id').substr(me._pathName.length);

			me._value(id);
			me.__hide();
		};


		this.__eOptions.onmouseover = function(e){
			var e = e || window.event;
			var elm = e.target || e.srcElement;

			if (elm.tagName != 'A') return;

			var id = elm.getAttribute('id').substr(me._pathName.length);

			if (typeof me.__tempValue != 'undefined'){

				if (me.__tempValue == id) return false;

				var elm = document.getElementById(me._pathName+me.__tempValue);
				if (elm) removecss(elm,'active');
			}

			me.__activate(id);

			me.__tempValue = id;
		};
		
		this._fill();
		return true;
	};
	


	_me.__destructor = function(x){
		if (this.__eOptions) this.__eOptions.parentNode.removeChild(this.__eOptions);
	};
	
	_me.__show = function(){

		if (this.__hideTimeout)
			window.clearTimeout(this.__hideTimeout);

		if (!this.__createOptionList())
		    return;

		this.__visibleOptions = true;

		var pos_scroll = this._getScrollPosition();
		var pos = getSize(this._main);
  			pos.x -= pos_scroll.x;
			pos.y -= pos_scroll.y;

		this.__eOptions.style.width = pos.w + 'px';
		this.__eOptions.style.top   = (pos.y+pos.h) + 'px';
		this.__eOptions.style.left 	= pos.x + 'px';


		this.__eButton.focus();

		this.__eOptions.style.display = 'block';

		if (count(this.__idTable)>10){
            this.__eOptions.style.overflow = 'auto';

            var size = 0;
			try{
            	size = getSize(this.__eOptions.getElementsByTagName('A')[0]).h*10;
			}
			catch(er){
				size = 153;
			}

			this.__eOptions.style.height = size +'px';
		}
		else{
            this.__eOptions.style.overflow = 'visible';
            this.__eOptions.style.height = 'auto';
		}

		this.__activate();

	};
	
	_me.__hide = function(){
		this.__visibleOptions = false;

		if (this.__eOptions)
			this.__eOptions.style.display = 'none';
	    
	    var elm;
		if((elm = document.getElementById(this._pathName+this.__tempValue))) removecss(elm,'active');
	};
	
	_me._getTextValue = function() {
		if (typeof this.__idTable[this.__value] == 'object')
		    return this.__idTable[this.__value][0];
		else
			return this.__idTable[this.__value];
	};
	/**
	 * atention in obj_charset is the same function!!
	 **/
	_me._value = function (sData,bNoEvn){
		// return
		if (typeof sData == 'undefined') return this.__value;

		var data = typeof this.__idTable[sData] == 'object'?this.__idTable[sData][0]:this.__idTable[sData];
			text = Is.String(data)?data.entityify():data;
		
		// set
        if (typeof data == 'undefined' || (this.__value == sData && this.__eButton.innerHTML == text)) return;
		this.__eButton.innerHTML = '<span>'+text+'</span>';

		this.__activate(sData);
		this.__value = sData;
		
		//inspect2({value:this.__value,type:typeof this.__value});

		if (typeof this.__idTable[sData] == 'object')
        	this.__eButton.className = 'obj_selectbutton '+this.__idTable[sData][1];

		if (!bNoEvn){
			// onchange event
	        if(this._onchange) this._onchange(null);
		    this.__exeEvent('onchange',null,{"owner":this});
	    }
	};
	
	_me.__activate = function(key){

		if (!this.__eOptions) return;
		
		var elm;
		if  (this.__tempValue){
			if ((elm = document.getElementById(this._pathName+this.__tempValue)))
				removecss(elm,'active');
			this.__tempValue = '';
		}
        
		if (this.__value && (elm = document.getElementById(this._pathName+this.__value))){
			if (typeof key == 'undefined'){
				addcss(elm,'active');

				// scroll to value
				if (elm.offsetTop>this.__eOptions.scrollTop+this.__eOptions.clientHeight || elm.offsetTop<this.__eOptions.scrollTop)
					this.__eOptions.scrollTop = elm.offsetTop - (this.__eOptions.clientHeight/2);

				return;
			}
			else
				removecss(elm,'active');
		}

		if (typeof key != 'undefined' && (elm = document.getElementById(this._pathName+key))){
			addcss(elm,'active');

			// scroll to value
			if (elm.offsetTop<this.__eOptions.scrollTop)
				this.__eOptions.scrollTop = elm.offsetTop+1;
			else
			if (elm.offsetTop+elm.offsetHeight>this.__eOptions.clientHeight+this.__eOptions.scrollTop)
				this.__eOptions.scrollTop = elm.offsetTop+elm.offsetHeight - this.__eOptions.clientHeight-1;
		}
	};
	
	_me._fill = function(aData){
		/* prepare data */
		if (aData)
			this.__idTable = aData

		if (this.__eOptions)
			this.__eOptions.innerHTML = '';

		if (!count(this.__idTable) || !this.__eOptions)
			return false;

		/* fill options */
		var eTmp,sTmp,title;
		
		for (var i in this.__idTable){

			eTmp = mkElement('a',{"href":''});
			
			if (typeof this.__idTable[i] == 'object'){
				title = this.__idTable[i][0];
				eTmp.className = this.__idTable[i][1];
			}
			else
				title = this.__idTable[i];

            eTmp.title = Is.String(title)?title.entityify():title;
			eTmp.innerHTML = '<span>'+(title?title.escapeHTML():'&nbsp;')+'</span>';
			
			eTmp.setAttribute('id',this._pathName+i);

			this.__eOptions.appendChild(eTmp);
			eTmp = null;
		}
		
		/* activate row */
		this.__activate();
	};

	_me._fillLang = function(aData){
		this.__idTable = {};
        for (var i in aData) {
			if (typeof aData[i] == 'object')
			    this.__idTable[i] = [getLang(aData[i][0]),aData[i][1]];
			else
				this.__idTable[i] = getLang(aData[i]);
		}
		this._fill();
	};

	_me._disabled = function(b){
		this.__disabled = (b?true:false);

		if (this.__disabled){
			addcss(this.__eButton,'obj_selectdisabled');
			this.__hide();
		}
		else
			removecss(this.__eButton,'obj_selectdisabled');
	};

	_me.__update = function (sDataSet){
		if (!sDataSet) return;

		if (this._listener == sDataSet)
			this._value(dataSet.get(this._listener,this._listenerPath));
		else
		if (this._listener_data == sDataSet)
			this._fill(dataSet.get(this._listener_data,this._listenerPath_data));
	};

	_me._listen_data = function(sDataSet,aDataPath){
		this._listener_data = sDataSet;
		if (typeof aDataPath == 'object') this._listenerPath_data = aDataPath;
		dataSet.obey(this,'_listener_data',sDataSet);
	};
